from django.core.urlresolvers import reverse_lazy
from django.views.generic import FormView, TemplateView

from .forms import UserCreationForm, UserActivationForm


class RegisterView(FormView):
    template_name = 'registration/register.html'
    form_class = UserCreationForm
    success_url = reverse_lazy('registered')

    def form_valid(self, form):
        form.save()
        return FormView.form_valid(self, form)

register = RegisterView.as_view()


class RegisteredView(TemplateView):
    template_name = 'registration/registered.html'

registered = RegisteredView.as_view()


class ActivateView(FormView):
    template_name = 'registration/activate.html'
    form_class = UserActivationForm
    success_url = reverse_lazy('activated')

    def form_valid(self, form):
        form.save()
        return FormView.form_valid(self, form)

    def get(self, request, *args, **kwargs):
        form = self.get_form_class()(kwargs)

        if form.is_valid():
            return self.form_valid(form)
        else:
            return self.form_invalid(form)

    def post(self, request, *args, **kwargs):
        return self.http_method_not_allowed(request, *args, **kwargs)

activate = ActivateView.as_view()


class ActivatedView(TemplateView):
    template_name = 'registration/activated.html'

activated = ActivatedView.as_view()



from datetime import datetime, timedelta
from hashlib import sha1
from random import random

from django.conf import settings
from django.core.mail import send_mail
from django.template.loader import render_to_string
from mongoengine import StringField, DateTimeField
from mongoengine.django.auth import User as BaseUser
from mongoengine.signals import pre_save


class User(BaseUser):

    activation_key = StringField()
    activation_due = DateTimeField()

    @staticmethod
    def ensure_inactive(klass, document):

        if document.id is None and not document.is_superuser:
            document.deactivate(save=False)

    @staticmethod
    def make_key():
        return sha1((settings.SECRET_KEY + str(random())).encode()).hexdigest()

    def deactivate(self, save=True, notify=True):
        self.is_active = False
        self.activation_key = self.make_key()
        self.activation_due = (
            datetime.utcnow() + timedelta(
                days=settings.ACCOUNT_ACTIVATION_DAYS))

        if save:
            self.save()

        if notify:
            c = {
                'site': settings.SITE,
                'user': self,
            }
            subject = render_to_string(
                'registration/email/activate_subject.txt', c)
            # Email subject *must not* contain newlines
            subject = ''.join(subject.splitlines())
            email = render_to_string(
                'registration/email/activate_email.txt', c)
            send_mail(subject, email, None, [self.email])

        return True

    def activate(self, activation_key, save=True):

        if self.activation_key != activation_key:
            return False
        if self.activation_due < datetime.utcnow():
            return False
        if self.is_active:
            return False

        self.is_active = True
        if save:
            self.save()
        return True

    def has_usable_password(self):
        """Dummy method for django.contrib.auth.forms.PasswordResetForm
        compatibility
        """
        return True

pre_save.connect(User.ensure_inactive, User)



from django.conf.urls import patterns, url


urlpatterns = patterns('regme.views',
    url('register/', 'register', {}, 'register'),
    url('registered/', 'registered', {}, 'registered'),
    url('activate/(?P<username>\w+)/(?P<activation_key>\w+)',
        'activate', {}, 'activate'),
    url('activated/', 'activated', {}, 'activated'),
)



import codecs

from setuptools import setup


with codecs.open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='regme',
    version='0.2.3',
    description='User registration and management library using MongoEngine',
    long_description=long_description,
    url='https://github.com/lig/regme',
    author='Serge Matveenko',
    author_email='s@matveenko.ru',
    license='Apache',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3',
        'Topic :: Database',
        'Framework :: Django',
    ],
    keywords='mongodb mongoengine django registration authentication',
    packages=['regme'],
    package_data={'regme': [
        'templates/registration/*.html',
        'templates/registration/email/*.txt']},
    install_requires=[
        'django',
        'mongoengine',
        'blinker',
    ],
)



from django import forms
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _


User = get_user_model()


class UserCreationForm(forms.Form):
    """
    A form that creates a user, with no privileges, from the given username and
    password.

    Based on `django.contrib.auth.forms.UserCreationForm`.
    """
    error_messages = {
        'duplicate_username': _("A user with that username already exists."),
        'password_mismatch': _("The two password fields didn't match."),
    }
    username = forms.RegexField(label=_("Username"), max_length=30,
        regex=r'^[\w.@+-]+$',
        help_text=_("Required. 30 characters or fewer. Letters, digits and "
                      "@/./+/-/_ only."),
        error_messages={
            'invalid': _("This value may contain only letters, numbers and "
                         "@/./+/-/_ characters.")})
    email = forms.EmailField(label=_("Email"), max_length=254,
        help_text=_("Required valid email. 254 characters or fewer."),
        error_messages={
            'invalid': _("This value may contain only valid email address.")})
    password1 = forms.CharField(label=_("Password"),
        widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"),
        widget=forms.PasswordInput,
        help_text=_("Enter the same password as above, for verification."))

    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            User._default_manager.get(username=username)
        except User.DoesNotExist:
            return username
        raise forms.ValidationError(
            self.error_messages['duplicate_username'],
            code='duplicate_username',
        )

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'],
                code='password_mismatch',
            )
        return password2

    def save(self):
        user = User._default_manager.create_user(
            username=self.cleaned_data['username'],
            email=self.cleaned_data['email'],
            password=self.cleaned_data["password1"])
        return user


class UserActivationForm(forms.Form):

    user = None
    error_messages = {
        'wrong_activation_key': _("Wrong activation key."),
        'user_already_activated': _("User already activated."),
    }
    username = forms.CharField(label=_("Username"), max_length=30,
        widget=forms.HiddenInput)
    activation_key = forms.CharField(label=_("Activation key"),
        widget=forms.HiddenInput)

    def clean(self):

        if self.cleaned_data.keys() == {'username', 'activation_key'}:

            try:
                user = User._default_manager.get(
                    username=self.cleaned_data['username'])
            except User.DoesNotExist:
                raise forms.ValidationError(
                    self.error_messages['wrong_activation_key'],
                    code='wrong_activation_key')

            if user.activate(self.cleaned_data['activation_key'], save=False):
                self.user = user
            elif user.is_active:
                raise forms.ValidationError(
                    self.error_messages['user_already_activated'],
                    code='user_already_activated')
            else:
                raise forms.ValidationError(
                    self.error_messages['wrong_activation_key'],
                    code='wrong_activation_key')

    def save(self):

        if self.user:
            self.user.save()
            return self.user
